part of 'application.dart';

class _ApplicationImpl extends StatefulWidget {
  final String name;
  final Set<AppRoute> routes;
  final String initialRoute;
  final OnUnknownRoute onUnknownRoute;
  final Set<LabelInterface>? labels;
  final ValueSetter<LabelInterface>? onLabel;
  final MaterialAppFactory materialAppFactory;

  const _ApplicationImpl(
      {Key? key,
      required this.name,
      required this.routes,
      required this.initialRoute,
      required this.onUnknownRoute,
      this.labels,
      this.onLabel,
      required this.materialAppFactory})
      : assert(labels == null || onLabel != null, 'onLabel must be provided along with labels'),
        super(key: key);

  @override
  _ApplicationImplState createState() => _ApplicationImplState();
}

class _ApplicationImplState extends State<_ApplicationImpl> implements Application {
  @override
  final MutableLiveData<bool> isInputMethodVisible = MutableLiveData(initialValue: false);
  @override
  final MutableLiveData<AppLifecycleState> lifecycle = MutableLiveData(initialValue: AppLifecycleState.detached);

  @override
  List<Locale> supportedLocales = const [];

  @override
  late Locale defaultLocale;

  @override
  Locale? get enforcedLocale => _enforcedLocale;

  @override
  set enforcedLocale(Locale? value) => setState(() => _enforcedLocale = value);

  late _AppBindingObserver _binding;
  late AppRouterDelegate _appRouterDelegate;
  GlobalKey<State<MaterialApp>> _key = GlobalKey();
  Locale? _enforcedLocale;

  @override
  void initState() {
    super.initState();
    _binding = _AppBindingObserver(this);
    WidgetsBinding.instance!.addObserver(_binding);
    _appRouterDelegate = AppRouterDelegate(widget.routes, widget.initialRoute, widget.onUnknownRoute);
  }

  @override
  void dispose() {
    WidgetsBinding.instance!.removeObserver(_binding);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final title = widget.name;
    final labels = widget.labels;
    final labelInterfaceDelegate = labels == null ? null : LabelInterfaceDelegate(labels, widget.onLabel!);
    final localizationsDelegates = <LocalizationsDelegate>[
      if (labelInterfaceDelegate != null) labelInterfaceDelegate,
      GlobalMaterialLocalizations.delegate,
      GlobalWidgetsLocalizations.delegate,
      GlobalCupertinoLocalizations.delegate,
    ];
    final systemLocales = WidgetsBinding.instance!.window.locales;
    supportedLocales = labels?.map((label) => label.locale).toList(growable: false) ?? systemLocales;
    defaultLocale = basicLocaleListResolution(systemLocales, supportedLocales);
    final root = widget.materialAppFactory(_key, _appRouterDelegate, _appRouterDelegate.provider,
        _appRouterDelegate.parser, title, _enforcedLocale, localizationsDelegates, supportedLocales);
    assert(() {
      WidgetsBinding.instance!.addPostFrameCallback((_) {
        final app = _key.currentWidget;
        if (app is! MaterialApp) throw StateError('MaterialAppFactory must set key parameter to the MaterialApp');
        if (app.routerDelegate != _appRouterDelegate ||
            app.routeInformationProvider != _appRouterDelegate.provider ||
            app.routeInformationParser != _appRouterDelegate.parser ||
            app.title != title ||
            (labelInterfaceDelegate != null && app.localizationsDelegates?.contains(labelInterfaceDelegate) != true) ||
            (labels != null && !app.supportedLocales.toSet().containsAll(supportedLocales)))
          throw StateError('MaterialAppFactory must set all given parameters');
      });
      return true;
    }());
    return root;
  }

  @override
  void restart() => setState(() => _key = GlobalKey());

  @override
  void exit() => die();
}
