import 'dart:async';

import 'package:collection/collection.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutterx_application/flutterx_application.dart';
import 'package:flutterx_application/src/platform/platform_io.dart'
    if (dart.library.html) 'package:flutterx_application/src/platform/platform_html.dart';
import 'package:flutterx_application/src/ui/system_ui_style.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

part 'app_route_activity.dart';
part 'app_route_information_parser.dart';
part 'app_route_information_provider.dart';
part 'app_router_delegate.dart';

typedef RouteArgs = Map<String, dynamic>;

typedef RouteWidgetBuilder = Widget Function(BuildContext context, RouteArgs args);

abstract class AppRoute<T> {
  final LocalKey? key;
  final String? name;
  final String location;
  final String? restorationId;

  const AppRoute._({
    required this.key,
    required this.name,
    required this.location,
    required this.restorationId,
  });

  Future<T> open(BuildContext context, {RouteArgs args = const {}, bool replace = false}) {
    final delegate = AppRouterDelegate.of(context);
    final page = _createPage(args);
    if (replace) {
      delegate._replaceRoute(page);
    } else {
      delegate._addRoute(page);
    }
    return page._result.future;
  }

  AppPage<T> createPage({RouteArgs args = const {}}) => _createPage(args);

  @factory
  AppPage<T> _createPage(RouteArgs args);

  @override
  String toString() => '$runtimeType($location)';

  static String _routeName(Object data) => data is String ? data : data.toString().snakeCase;

  static String _routeLocation(Object data) => Navigator.defaultRouteName + _routeName(data);
}

abstract class AppPage<T> extends Page {
  final AppRoute<T> route;

  @override
  RouteArgs get arguments => super.arguments as RouteArgs;

  AppPage._({required this.route, required RouteArgs args})
      : super(key: route.key, name: route.name, arguments: args, restorationId: route.restorationId);

  final Completer<T> _result = Completer();

  @override
  Route<T> createRoute(BuildContext context) =>
      _createRoute(context)..popped.then(_result.complete, onError: _result.completeError);

  @factory
  Route<T> _createRoute(BuildContext context);

  @override
  String toString() => '$runtimeType($name)';
}
