part of 'app_route.dart';

class ActivityRoute<T> extends AppRoute<T> {
  final bool maintainState;
  final bool fullscreenDialog;
  final SystemUiStyleBuilder systemUiStyle;
  final RouteWidgetBuilder builder;

  ActivityRoute.builder({
    LocalKey? key,
    required Object name,
    String? location,
    String? restorationId,
    this.maintainState = true,
    this.fullscreenDialog = false,
    this.systemUiStyle = Application.defaultSystemUiStyle,
    required this.builder,
  }) : super._(
            location: location ?? AppRoute._routeLocation(name),
            key: key,
            name: AppRoute._routeName(name),
            restorationId: restorationId);

  @override
  ActivityPage<T> _createPage(RouteArgs args) => ActivityPage._(route: this, args: args);
}

class ActivityPage<T> extends AppPage<T> {
  @override
  ActivityRoute<T> get route => super.route as ActivityRoute<T>;

  ActivityPage._({required ActivityRoute<T> route, required RouteArgs args}) : super._(route: route, args: args);

  @override
  Route<T> _createRoute(BuildContext context) => MaterialPageRoute(
      builder: (context) =>
          AnnotatedSystemUIStyle(style: route.systemUiStyle(context), child: route.builder(context, arguments)),
      settings: this,
      maintainState: route.maintainState,
      fullscreenDialog: route.fullscreenDialog);

  @optionalTypeArgs
  static ActivityPage<T> of<T>(BuildContext context) =>
      AppRouterDelegate.of(context).currentConfiguration as ActivityPage<T>;
}
